import { jsx as _jsx } from "react/jsx-runtime";
import React, { memo, useState, useMemo, useEffect, useRef, useContext } from 'react';
import { ApolloProvider } from '@apollo/client';
import { generateApolloClient } from '@deep-foundation/hasura/client.js';
import { debug } from './debug.js';
const moduleLog = debug.extend('apollo-client-regenerator');
export const defaultRegenerateApolloClient = (options) => {
    if (!generateApolloClient)
        throw new Error("!generateApolloClient");
    return generateApolloClient(options);
};
export const ApolloClientRegeneratorContext = React.createContext({ regenerate: () => { } });
export function useApolloClientRegenerator() {
    return useContext(ApolloClientRegeneratorContext);
}
/**
 * Recreate ApolloClient on mount, and on each token or options changes. Make sure you memo it and dont update options object without need.
 */
export const ApolloClientRegenerator = memo(function ApolloClientRegenerator({ options, children = null, useStateApolloClient = useState, regenerateApolloClient = defaultRegenerateApolloClient, }) {
    const log = moduleLog.extend(ApolloClientRegenerator.name);
    const initialApolloClient = useMemo(() => regenerateApolloClient(options), []);
    log({ initialApolloClient });
    const [apolloClient, setApolloClient] = useStateApolloClient(initialApolloClient);
    log({ apolloClient, setApolloClient });
    const mountedRef = useRef(false);
    useEffect(() => {
        if (!mountedRef.current)
            mountedRef.current = true;
        else
            setApolloClient(regenerateApolloClient(options));
    }, [options]);
    const context = useMemo(() => ({
        regenerate: () => {
            setApolloClient(regenerateApolloClient(options));
        },
    }), [options]);
    return _jsx(ApolloClientRegeneratorContext.Provider, { value: context, children: _jsx(ApolloProvider, { client: apolloClient, children: children }) });
});
